config = {
    "version": "1.0",
    "source": {
        "type": "FTP",
        "ftp": {
            "base_url": "ladsweb.nascom.nasa.gov",
            "data_dir": "/allData/5/"
        }
    },
    "target": {
        "target_dir": "/home/kalimaha/Desktop/MODIS",
        "folders": [
            {
                "folder_name": "{{product}}",
                "folders": [
                    {
                        "folder_name": "{{year}}",
                        "folders": [
                            {
                                "folder_name": "{{day}}"
                            }
                        ]
                    }
                ]
            }
        ]
    },
    "bands": [
        {
            "index": 1,
            "label": "NDVI"
        },
        {
            "index": 2,
            "label": "EVI"
        }
    ],
    "subfolders": {
        "output": "OUTPUT"
    },
    "processing": [
        {
            "band": 1,
            "process": [
                {
                    "extract_bands": ""
                }
            ]
        },
        {
            "band": 1,
            "output_file_name": "merge",
            "process": [
                {
                    "gdal_merge": {
                        "prefix": "gdal_merge_",
                        "extension": "tif"
                    }
                }
            ]
        },
        {
            "band": 1,
            "output_file_name": "final",
            "process": [
                {
                    "get_pixel_size" : "{{PIXEL_SIZE}}"
                },
                {
                    "gdalwarp": {
                        "opt": {
                            "-multi": "",
                            "-overwrite": "",
                            "-of": "GTiff",
                            "-s_srs": "'+proj=sinu +R=6371007.181 +nadgrids=@null +wktext'",
                            "-co": "'TILED=YES'",
                            "-t_srs": "EPSG:3857",
                            "-srcnodata": -3000,
                            "-dstnodata": -3000
                        },
                        "prefix": "gdalwarp_",
                        "extension": "tif"
                    }
                }
            ]
        },
        {
            "band": 1,
            "process": [
                {
                    "gdaladdo": {
                        "parameters": {
                            "-r": "average"
                        },
                        "overviews_levels": "2 4 8 16"
                    }
                }
            ]
        }
    ],
    "analysis": {
        "create_avg": True,
        "create_da": True,
        "stats": {
            "enable": True,
            "scripts": [
                {
                    "file": "sadc.json",
                    "description": {
                        "en": "SADC"
                    }
                }
            ]
        }
    },
    "services_base_url": "http://168.202.28.57:5005/browse/",
    "services": {
        "layers": {
            "id": "list_layers",
            "path": "modis/{product}/{year}/{day}",
            "parameters": [
                {
                    "parameter_name": "product",
                    "parameter_value": "list_products"
                },
                {
                    "parameter_name": "year",
                    "parameter_value": "list_years_from"
                },
                {
                    "parameter_name": "day",
                    "parameter_value": "$create_day_of_the_year"
                }
            ],
            "payload": {
                "fields": ["code", "label", "size"],
                "id": "code",
                "label": "label"
            },
            "description": {
                "en": "Available Layers",
                "it": "Layers Disponibili",
                "pt": "Layers Disponiveis"
            },
            "selection_type": "multiple"
        },
        "download": {
            "__type": "standard",
            "payload": {
                "filesystem_structure": {
                    "type": "object",
                    "parameters": [
                        {
                            "parameter_name": "product",
                            "parameter_value": "list_products"
                        },
                        {
                            "parameter_name": "year",
                            "parameter_value": "list_years_from"
                        },
                        {
                            "parameter_name": "day",
                            "parameter_value": "$create_day_of_the_year"
                        }
                    ]
                },
                "file_paths_and_sizes": {
                    "type": "rest_response"
                },
                "tab_id": {
                    "type": "string",
                    "parameters": [
                        "tab_",
                        "$CURRENT_INDEX"
                    ]
                }
            }
        },
        "countries": {
            "id": "list_gaul2modis",
            "path": "modis/countries",
            "parameters": [],
            "payload": {
                "fields": ["gaul_code", "gaul_label", "from_h", "to_h", "from_v", "to_v"],
                "id": "gaul_code",
                "label": "gaul_label"
            },
            "description": {
                "en": "Countries",
                "it": "Paesi",
                "pt": "Paises"
            },
            "selection_type": "multiple"
        },
        "time_range": {
            "year": "list_years_from",
            "from": "list_days_from",
            "to": "list_days_to",
            "step": 16
        },
        "filters": [
            {
                "id": "list_products",
                "path": "modis",
                "parameters": [],
                "payload": {
                    "fields": ["code", "label"],
                    "id": "code",
                    "label": "label"
                },
                "description": {
                    "en": "Available Products",
                    "it": "Prodotti Disponibili",
                    "pt": "Produtos Disponiveis"
                },
                "selection_type": "single",
                "services": [
                    {
                        "id": "list_years_from",
                        "path": "modis/{product}",
                        "parameters": [
                            {
                                "parameter_name": "product",
                                "parameter_value": "list_products"
                            }
                        ],
                        "payload": {
                            "fields": ["code", "label"],
                            "id": "code",
                            "label": "label"
                        },
                        "description": {
                            "en": "Available Years",
                            "it": "Anni Disponibili",
                            "pt": "Anos Disponiveis"
                        },
                        "selection_type": "single",
                        "services": [
                            {
                                "id": "list_days_from",
                                "path": "modis/{product}/{year}",
                                "parameters": [
                                    {
                                        "parameter_name": "product",
                                        "parameter_value": "list_products"
                                    },
                                    {
                                        "parameter_name": "year",
                                        "parameter_value": "list_years_from"
                                    }
                                ],
                                "payload": {
                                    "fields": ["code", "label"],
                                    "id": "code",
                                    "label": "label"
                                },
                                "description": {
                                    "en": "Available Days - From",
                                    "it": "Giorni Disponibili - Da",
                                    "pt": "Dias Disponiveis - Desde"
                                },
                                "selection_type": "single"
                            },
                            {
                                "id": "list_days_to",
                                "path": "modis/{product}/{year}",
                                "parameters": [
                                    {
                                        "parameter_name": "product",
                                        "parameter_value": "list_products"
                                    },
                                    {
                                        "parameter_name": "year",
                                        "parameter_value": "list_years_from"
                                    }
                                ],
                                "payload": {
                                    "fields": ["code", "label"],
                                    "id": "code",
                                    "label": "label"
                                },
                                "description": {
                                    "en": "Available Days - To",
                                    "it": "Giorni Disponibili - A",
                                    "pt": "Dias Disponiveis - Ate"
                                },
                                "selection_type": "single"
                            }
                        ]
                    }
                ]
            }
        ]
    }
}


import json
from flask import Blueprint
from flask import Response
from flask import request
from flask.ext.cors import cross_origin
from pgeo.error.custom_exceptions import PGeoException
from pgeomodis.core import modis_core as m


modis = Blueprint('modis', __name__)


@modis.route('/')
@cross_origin(origins='*')
def list_products_service():
    try:
        out = m.get_modis_product_table()
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/<product_name>/')
@cross_origin(origins='*')
def list_years_service(product_name):
    try:
        out = m.list_years(product_name)
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/<product_name>/<year>/')
@cross_origin(origins='*')
def list_days_service(product_name, year):
    try:
        out = m.list_days(product_name, year)
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/<product_name>/<year>/<day>/')
@cross_origin(origins='*')
def list_layers_service(product_name, year, day):
    try:
        out = m.list_layers(product_name, year, day)
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/<product_name>/<year>/<day>/<from_h>/<to_h>/<from_v>/<to_v>/')
@cross_origin(origins='*')
def list_layers_subset_service(product_name, year, day, from_h, to_h, from_v, to_v):
    try:
        out = m.list_layers_subset(product_name, year, day, from_h, to_h, from_v, to_v)
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/<product_name>/<year>/<day>/<countries>/')
@cross_origin(origins='*')
def list_layers_countries_subset_service(product_name, year, day, countries):
    try:
        out = m.list_layers_countries_subset(product_name, year, day, countries)
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


@modis.route('/countries/')
@cross_origin(origins='*')
def list_countries():
    print request.host_url
    try:
        out = m.list_countries()
        return Response(json.dumps(out), content_type='application/json; charset=utf-8')
    except PGeoException, e:
        raise PGeoException(e.get_message(), e.get_status_code())


from ftplib import FTP
from pgeo.error.custom_exceptions import PGeoException
from pgeo.error.custom_exceptions import errors
from pgeo.utils.date import day_of_the_year_to_date
from bs4 import BeautifulSoup
import urllib
from pgeomodis.config.gaul2modis import map
from pgeomodis.config.modis_config import config as conf


def get_modis_product_table():

    sock = urllib.urlopen('https://lpdaac.usgs.gov/products/modis_products_table')
    html = sock.read()
    sock.close()
    soup = BeautifulSoup(html)

    tables = soup.findAll('table')
    table = tables[len(tables) - 1]
    tbody = table.find('tbody')
    trs = tbody.findAll('tr')

    products = []
    keys = ['code', 'platform', 'modis_data_product', 'raster_type', 'spatial_resolution', 'temporal_resolution']

    for tr in trs:
        p = {}
        counter = 0
        tds = tr.findAll('td')
        for td in tds:
            text = ''.join(td.find(text=True)).strip().replace('\n', '')
            if counter == 0:
                text = td.find('a').find(text=True)
            p[keys[counter]] = text
            counter += 1
        products.append(p)

    return products


def list_products():
    """
    List all the available MODIS products.
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            l = ftp.nlst()
            l.sort()
            out = []
            for s in l:
                out.append({'code': s, 'label': s})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_years(product_name):
    """
    List all the available years for a given MODIS product.
    @param product_name: Code of MODIS product, e.g. 'MOD13Q1'
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(product_name.upper())
            l = ftp.nlst()
            l.sort(reverse=True)
            out = []
            for s in l:
                try:
                    float(s)
                    out.append({'code': s, 'label': s})
                except ValueError:
                    pass
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_days(product_name, year):
    """
    List all the available days for a given MODIS product and year.
    @param product_name: Code of MODIS product, e.g. 'MOD13Q1'
    @param year: e.g. '2010'
    @return: An array of code/label objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(product_name.upper())
            ftp.cwd(year)
            l = ftp.nlst()
            l.sort()
            out = []
            for s in l:
                date = day_of_the_year_to_date(s, year).strftime('%d %B')
                out.append({'code': s, 'label': date})
            ftp.quit()
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_layers(product_name, year, day):
    """
    List all the available layers for a given MODIS product, year and day.
    @param product_name: Code of MODIS product, e.g. 'MOD13Q1'
    @param year: e.g. '2010'
    @param day: Day of the year, three digits, e.g. '017'
    @return: An array of code/label/size objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(product_name.upper())
            ftp.cwd(year)
            ftp.cwd(day)
            ls = []
            ftp.retrlines('MLSD', ls.append)
            ftp.quit()
            out = []
            buffer = []
            for line in ls:
                try:
                    start = line.index('Size=')
                    end = line.index(';', start)
                    size = line[start + len('Size='):end]
                    start = line.index(product_name.upper())
                    file_name = line[start:]
                    if file_name not in buffer:
                        buffer.append(file_name)
                        file_path = 'ftp://' + conf['source']['ftp']['base_url'] + conf['source']['ftp']['data_dir']
                        file_path += product_name.upper() + '/' + year + '/' + day + '/'
                        file_path += line[start:]
                        h = file_name[2 + file_name.index('.h'):4 + file_name.index('.h')]
                        v = file_name[1 + file_name.index('v'):3 + file_name.index('v')]
                        label = 'H ' + h + ', V ' + v + ' (' + str(round((float(size) / 1000000), 2)) + ' MB)'
                        out.append({
                            'file_name': file_name,
                            'file_path': file_path,
                            'label': label,
                            'size': size
                        })
                except:
                    pass
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except:
        raise PGeoException(errors[511], status_code=511)


def list_layers_subset(product_name, year, day, from_h, to_h, from_v, to_v):
    """
    List all the available layers for a given MODIS product, year and day.
    @param product_name: Code of MODIS product, e.g. 'MOD13Q1'
    @param year: e.g. '2010'
    @param day: Day of the year, three digits, e.g. '017'
    @param from_h: e.g. '05'
    @param to_h: e.g. '05'
    @param from_v: e.g. '05'
    @param to_v: e.g. '05'
    @return: An array of code/label/size objects.
    """
    try:
        if conf['source']['type'] == 'FTP':
            ftp = FTP(conf['source']['ftp']['base_url'])
            ftp.login()
            ftp.cwd(conf['source']['ftp']['data_dir'])
            ftp.cwd(product_name.upper())
            ftp.cwd(year)
            ftp.cwd(day)
            ls = []
            ftp.retrlines('MLSD', ls.append)
            ftp.quit()
            out = []
            buffer = []
            for line in ls:
                try:
                    start = line.index('Size=')
                    end = line.index(';', start)
                    size = line[start + len('Size='):end]
                    start = line.index(product_name.upper())
                    file_name = line[start:]
                    if file_name not in buffer:
                        buffer.append(file_name)
                        if is_layer_in_the_range(file_name, from_h, to_h, from_v, to_v):
                            file_path = 'ftp://' + conf['source']['ftp']['base_url'] + conf['source']['ftp']['data_dir']
                            file_path += product_name.upper() + '/' + year + '/' + day + '/'
                            file_path += line[start:]
                            h = file_name[2 + file_name.index('.h'):4 + file_name.index('.h')]
                            v = file_name[1 + file_name.index('v'):3 + file_name.index('v')]
                            label = 'H ' + h + ', V ' + v + ' (' + str(round((float(size) / 1000000), 2)) + ' MB)'
                            out.append({
                                'file_name': file_name,
                                'file_path': file_path,
                                'label': label,
                                'size': size
                            })
                except:
                    pass
            return out
        else:
            raise PGeoException(errors[512], status_code=512)
    except Exception, e:
        raise PGeoException(str(e), status_code=500)


def is_layer_in_the_range(file_name, from_h, to_h, from_v, to_v):
    h_idx = len('.h') + file_name.index('.h')
    h = int(file_name[h_idx:(2 + h_idx)])
    v_idx = len(str(h) + 'v') + file_name.index(str(h) + 'v')
    v = int(file_name[v_idx:(2 + v_idx)])
    if int(from_h) <= h <= int(to_h) and int(from_v) <= v <= int(to_v):
        return True
    return False


def list_countries():
    return map


def list_layers_countries_subset(product_name, year, day, countries):
    """
    List all the available layers for a given MODIS product, year and day.
    @param product_name: Code of MODIS product, e.g. 'MOD13Q1'
    @param year: e.g. '2010'
    @param day: Day of the year, three digits, e.g. '017'
    @param countries: GAUL codes, comma separated e.g. '18,25,34'
    @type countries: String, comma separated
    @return: An array of code/label/size objects.
    """
    countries_list = countries.split(',')
    out = []
    clean_out = []
    file_names_buffer = []
    try:
        for g2m in map:
            if g2m['gaul_code'] in countries_list:
                from_h = g2m['from_h']
                to_h = g2m['to_h']
                from_v = g2m['from_v']
                to_v = g2m['to_v']
                tmp = list_layers_subset(product_name, year, day, from_h, to_h, from_v, to_v)
                out += tmp
        for tmp in out:
            if tmp['file_name'] not in file_names_buffer:
                file_names_buffer.append(tmp['file_name'])
                clean_out.append(tmp)
        return clean_out
    except Exception, e:
        raise PGeoException(e.get_message(), e.get_status_code())


from setuptools import setup
from setuptools import find_packages


setup(
    name='PGeoMODIS',
    version='0.1.6',
    author='Simone Murzilli; Guido Barbaglia',
    author_email='geobrickspy@gmail.com',
    packages=find_packages(),
    install_requires=[
        'flask'
    ],
    url='http://pypi.python.org/pypi/PGeoMODIS/',
    license='LICENSE.txt',
    description='MODIS plug-in for PGeo.',
    long_description=open('README.md').read()
)
